@@ -15,21 +15,27 @@ also return a dict of list of dicts
 """
 
 import collections
+import itertools
+import re
+import requests
+from pyparsing import ParseException
 
 from rdflib import Variable, Graph, BNode, URIRef, Literal
+from six import iteritems, itervalues
 
 from rdflib.plugins.sparql import CUSTOM_EVALS
 from rdflib.plugins.sparql.parserutils import value
 from rdflib.plugins.sparql.sparql import (
-    QueryContext, AlreadyBound, FrozenBindings, SPARQLError)
+    QueryContext, AlreadyBound, FrozenBindings, Bindings, SPARQLError)
 from rdflib.plugins.sparql.evalutils import (
     _filter, _eval, _join, _diff, _minus, _fillTemplate, _ebv, _val)
 
 from rdflib.plugins.sparql.aggregates import Aggregator
 from rdflib.plugins.sparql.algebra import Join, ToMultiSet, Values
+from rdflib.plugins.sparql import parser
+
 
 def evalBGP(ctx, bgp):
-
     """
     A basic graph pattern
     """
@@ -94,7 +100,7 @@ def evalLazyJoin(ctx, join):
     for a in evalPart(ctx, join.p1):
         c = ctx.thaw(a)
         for b in evalPart(c, join.p2):
-            yield b.merge(a) # merge, as some bindings may have been forgotten
+            yield b.merge(a)  # merge, as some bindings may have been forgotten
 
 
 def evalJoin(ctx, join):
@@ -111,13 +117,10 @@ def evalJoin(ctx, join):
 
 
 def evalUnion(ctx, union):
-    res = set()
     for x in evalPart(ctx, union.p1):
-        res.add(x)
         yield x
     for x in evalPart(ctx, union.p2):
-        if x not in res:
-            yield x
+        yield x
 
 
 def evalMinus(ctx, minus):
@@ -143,8 +146,8 @@ def evalLeftJoin(ctx, join):
             # even without prior bindings...
             p1_vars = join.p1._vars
             if p1_vars is None \
-            or not any(_ebv(join.expr, b) for b in
-                       evalPart(ctx.thaw(a.remember(p1_vars)), join.p2)):
+                or not any(_ebv(join.expr, b) for b in
+                           evalPart(ctx.thaw(a.remember(p1_vars)), join.p2)):
 
                 yield a
 
@@ -209,7 +212,7 @@ def evalMultiset(ctx, part):
 def evalPart(ctx, part):
 
     # try custom evaluation functions
-    for name, c in list(CUSTOM_EVALS.items()):
+    for name, c in CUSTOM_EVALS.items():
         try:
             return c(ctx, part)
         except NotImplementedError:
@@ -262,18 +265,90 @@ def evalPart(ctx, part):
         return evalConstructQuery(ctx, part)
 
     elif part.name == 'ServiceGraphPattern':
-        raise Exception('ServiceGraphPattern not implemented')
+        return evalServiceQuery(ctx, part)
+        #raise Exception('ServiceGraphPattern not implemented')
 
     elif part.name == 'DescribeQuery':
         raise Exception('DESCRIBE not implemented')
 
     else:
-        # import pdb ; pdb.set_trace()
         raise Exception('I dont know: %s' % part.name)
 
+def evalServiceQuery(ctx, part):
+    res = {}
+    match = re.match('^service <(.*)>[ \n]*{(.*)}[ \n]*$',
+                     part.get('service_string', ''), re.DOTALL)
+
+    if match:
+        service_url = match.group(1)
+        service_query = _buildQueryStringForServiceCall(ctx, match)
+
+        query_settings = {'query': service_query,
+                          'output': 'json'}
+        headers = {'accept' : 'application/sparql-results+json',
+                          'user-agent': 'rdflibForAnUser'}
+        # GET is easier to cache so prefer that if the query is not to long
+        if len(service_query) < 600:
+            response = requests.get(service_url, params=query_settings, headers=headers)
+        else:
+            response = requests.post(service_url, params=query_settings, headers=headers)
+        if response.status_code == 200:
+            json = response.json();
+            variables = res["vars_"] = json['head']['vars']
+            # or just return the bindings?
+            res = json['results']['bindings']
+            if len(res) > 0:
+                for r in res:
+                    for bound in _yieldBindingsFromServiceCallResult(ctx, r, variables):
+                        yield bound
+        else:
+            raise Exception("Service: %s responded with code: %s", service_url, response.status_code);
+
+
+"""
+    Build a query string to be used by the service call. 
+    It is supposed to pass in the existing bound solutions.
+    Re-adds prefixes if added and sets the base.
+    Wraps it in select if needed.
+"""
+def _buildQueryStringForServiceCall(ctx, match):
+
+    service_query = match.group(2)
+    try:
+        parser.parseQuery(service_query)
+    except ParseException:
+        # This could be because we don't have a select around the service call.
+        service_query = 'SELECT REDUCED * WHERE {' + service_query + '}'
+        for p in ctx.prologue.namespace_manager.store.namespaces():
+            service_query = 'PREFIX ' + p[0] + ':' + p[1].n3() + ' ' + service_query
+        # re add the base if one was defined
+        base = ctx.prologue.base
+        if base is not None and len(base) > 0:
+            service_query = 'BASE <' + base + '> ' + service_query
+    sol = ctx.solution();
+    if len(sol) > 0:
+        variables = ' '.join(map(lambda v:v.n3(), sol))
+        variables_bound = ' '.join(map(lambda v: ctx.get(v).n3(), sol))
+        service_query = service_query + 'VALUES (' + variables + ') {(' + variables_bound + ')}'
+    return service_query
+
+
+def _yieldBindingsFromServiceCallResult(ctx, r, variables):
+    res_dict = {}
+    for var in variables:
+        if var in r and r[var]:
+            if r[var]['type'] == "uri":
+                res_dict[Variable(var)] = URIRef(r[var]["value"])
+            elif r[var]['type'] == "bnode":
+                res_dict[Variable(var)] = BNode(r[var]["value"])
+            elif r[var]['type'] == "literal" and 'datatype' in r[var]:
+                res_dict[Variable(var)] = Literal(r[var]["value"], datatype=r[var]['datatype'])
+            elif r[var]['type'] == "literal" and 'xml:lang' in r[var]:
+                res_dict[Variable(var)] = Literal(r[var]["value"], lang=r[var]['xml:lang'])
+    yield FrozenBindings(ctx, res_dict)
+
 
 def evalGroup(ctx, group):
-
     """
     http://www.w3.org/TR/sparql11-query/#defn_algGroup
     """
@@ -302,7 +377,7 @@ def evalAggregateJoin(ctx, agg):
             res[k].update(row)
 
     # all rows are done; yield aggregated values
-    for aggregator in res.values():
+    for aggregator in itervalues(res):
         yield FrozenBindings(ctx, aggregator.get_bindings())
 
     # there were no matches
@@ -323,25 +398,9 @@ def evalOrderBy(ctx, part):
 
 
 def evalSlice(ctx, slice):
-    # import pdb; pdb.set_trace()
     res = evalPart(ctx, slice.p)
-    i = 0
-    try:
-        while i < slice.start:
-            next(res)
-            i += 1
-    except StopIteration:
-        return
-    i = 0
-    for x in res:
-        i += 1
-        if slice.length is None:
-            yield x
-        else:
-            if i <= slice.length:
-                yield x
-            else:
-                break
+
+    return itertools.islice(res, slice.start, slice.start + slice.length if slice.length is not None else None)
 
 
 def evalReduced(ctx, part):
@@ -372,7 +431,7 @@ def evalReduced(ctx, part):
             # forget last position of row
             mru_queue.remove(row)
         else:
-            #row seems to be new
+            # row seems to be new
             yield row
             mru_set.add(row)
             if len(mru_set) > MAX:
@@ -439,7 +498,7 @@ def evalConstructQuery(ctx, query):
 
 def evalQuery(graph, query, initBindings, base=None):
 
-    initBindings = dict( ( Variable(k),v ) for k,v in initBindings.items() )
+    initBindings = dict((Variable(k), v) for k, v in iteritems(initBindings))
 
     ctx = QueryContext(graph, initBindings=initBindings)
 
