@@ -1,15 +1,20 @@
+from __future__ import absolute_import
+
 import collections
 import itertools
 import datetime
 
+import isodate
+from six import text_type, iteritems
+
+from rdflib.compat import Mapping, MutableMapping
 from rdflib.namespace import NamespaceManager
 from rdflib import Variable, BNode, Graph, ConjunctiveGraph, URIRef, Literal
 from rdflib.term import Node
 
-from .parserutils import CompValue
+from rdflib.plugins.sparql.parserutils import CompValue
 
 import rdflib.plugins.sparql
-from rdflib.plugins.sparql.compat import Mapping, MutableMapping
 
 
 class SPARQLError(Exception):
@@ -24,6 +29,7 @@ class NotBoundError(SPARQLError):
 
 class AlreadyBound(SPARQLError):
     """Raised when trying to bind a variable that is already bound!"""
+
     def __init__(self):
         SPARQLError.__init__(self)
 
@@ -83,10 +89,10 @@ class Bindings(MutableMapping):
             d = d.outer
 
     def __str__(self):
-        return "Bindings({"+", ".join((k, self[k]) for k in self)+"})"
+        return "Bindings({" + ", ".join((k, self[k]) for k in self) + "})"
 
     def __repr__(self):
-        return str(self)
+        return text_type(self)
 
 
 class FrozenDict(Mapping):
@@ -96,6 +102,7 @@ class FrozenDict(Mapping):
     Taken from http://stackoverflow.com/a/2704866/81121
 
     """
+
     def __init__(self, *args, **kwargs):
         self._d = dict(*args, **kwargs)
         self._hash = None
@@ -117,14 +124,14 @@ class FrozenDict(Mapping):
         # urge to optimize when it will gain improved algorithmic performance.
         if self._hash is None:
             self._hash = 0
-            for key, value in self.items():
+            for key, value in iteritems(self):
                 self._hash ^= hash(key)
                 self._hash ^= hash(value)
         return self._hash
 
     def project(self, vars):
         return FrozenDict(
-            (x for x in self.items() if x[0] in vars))
+            (x for x in iteritems(self) if x[0] in vars))
 
     def disjointDomain(self, other):
         return not bool(set(self).intersection(other))
@@ -141,7 +148,7 @@ class FrozenDict(Mapping):
 
     def merge(self, other):
         res = FrozenDict(
-            itertools.chain(iter(self.items()), iter(other.items())))
+            itertools.chain(iteritems(self), iteritems(other)))
 
         return res
 
@@ -166,15 +173,18 @@ class FrozenBindings(FrozenDict):
         if not type(key) in (BNode, Variable):
             return key
 
-        return self._d[key]
+        if key not in self._d:
+            return self.ctx.initBindings[key]
+        else:
+            return self._d[key]
 
     def project(self, vars):
         return FrozenBindings(
-            self.ctx, (x for x in self.items() if x[0] in vars))
+            self.ctx, (x for x in iteritems(self) if x[0] in vars))
 
     def merge(self, other):
         res = FrozenBindings(
-            self.ctx, itertools.chain(iter(self.items()), iter(other.items())))
+            self.ctx, itertools.chain(iteritems(self), iteritems(other)))
 
         return res
 
@@ -196,15 +206,23 @@ class FrozenBindings(FrozenDict):
         return a frozen dict only of bindings made in self
         since before
         """
-        if not _except : _except = []
+        if not _except:
+            _except = []
+
         # bindings from initBindings are newer forgotten
-        return FrozenBindings(self.ctx, (x for x in self.items() if x[0] in _except or x[0] in self.ctx.initBindings or before[x[0]] is None))
+        return FrozenBindings(
+            self.ctx, (
+                x for x in iteritems(self) if (
+                    x[0] in _except or
+                    x[0] in self.ctx.initBindings or
+                    before[x[0]] is None)))
 
     def remember(self, these):
         """
         return a frozen dict only of bindings in these
         """
-        return FrozenBindings(self.ctx, (x for x in self.items() if x[0] in these))
+        return FrozenBindings(
+            self.ctx, (x for x in iteritems(self) if x[0] in these))
 
 
 class QueryContext(object):
@@ -216,7 +234,8 @@ class QueryContext(object):
     def __init__(self, graph=None, bindings=None, initBindings=None):
         self.initBindings = initBindings
         self.bindings = Bindings(d=bindings or [])
-        if initBindings: self.bindings.update(initBindings)
+        if initBindings:
+            self.bindings.update(initBindings)
 
         if isinstance(graph, ConjunctiveGraph):
             self._dataset = graph
@@ -229,7 +248,7 @@ class QueryContext(object):
             self.graph = graph
 
         self.prologue = None
-        self.now = datetime.datetime.now()
+        self.now = datetime.datetime.now(isodate.tzinfo.UTC)
 
         self.bnodes = collections.defaultdict(BNode)
 
@@ -267,7 +286,7 @@ class QueryContext(object):
             except:
                 raise Exception(
                     "Could not load %s as either RDF/XML, N3 or NTriples" % (
-                    source))
+                        source))
 
         if not rdflib.plugins.sparql.SPARQL_LOAD_GRAPHS:
             # we are not loading - if we already know the graph
@@ -303,10 +322,10 @@ class QueryContext(object):
         if vars:
             return FrozenBindings(
                 self, ((k, v)
-                       for k, v in self.bindings.items()
+                       for k, v in iteritems(self.bindings)
                        if k in vars))
         else:
-            return FrozenBindings(self, iter(self.bindings.items()))
+            return FrozenBindings(self, iteritems(self.bindings))
 
     def __setitem__(self, key, value):
         if key in self.bindings and self.bindings[key] != value:
@@ -361,7 +380,6 @@ class Prologue(object):
         self.namespace_manager.bind(prefix, uri, replace=True)
 
     def absolutize(self, iri):
-
         """
         Apply BASE / PREFIXes to URIs
         (and to datatypes in Literals)
