@@ -13,98 +13,19 @@ ORDERBY = 'ORDER BY'
 
 import re
 import collections
-import warnings
-import contextlib
 
-try:
-    from SPARQLWrapper import SPARQLWrapper, XML, POST, GET, URLENCODED, POSTDIRECTLY
-except ImportError:
-    raise Exception(
-        "SPARQLWrapper not found! SPARQL Store will not work." +
-        "Install with 'easy_install SPARQLWrapper'")
+from .sparqlconnector import SPARQLConnector
 
-from rdflib.compat import etree, etree_register_namespace
 from rdflib.plugins.stores.regexmatching import NATIVE_REGEX
 
 from rdflib.store import Store
-from rdflib.query import Result
-from rdflib import Variable, Namespace, BNode, URIRef, Literal
+from rdflib import Variable, BNode
 from rdflib.graph import DATASET_DEFAULT_GRAPH_ID
 from rdflib.term import Node
 
-class NSSPARQLWrapper(SPARQLWrapper):
-    nsBindings = {}
-
-    def setNamespaceBindings(self, bindings):
-        """
-        A shortcut for setting namespace bindings that will be added
-        to the prolog of the query
-
-        @param bindings: A dictionary of prefixs to URIs
-        """
-        self.nsBindings.update(bindings)
-
-    def setQuery(self, query):
-        """
-        Set the SPARQL query text. Note: no check is done on the
-        validity of the query (syntax or otherwise) by this module,
-        except for testing the query type (SELECT, ASK, etc).
-
-        Syntax and validity checking is done by the SPARQL service itself.
-
-        @param query: query text
-        @type query: string
-        @bug: #2320024
-        """
-        self.queryType = self._parseQueryType(query)
-        self.queryString = self.injectPrefixes(query)
-
-    def injectPrefixes(self, query):
-        prefixes = list(self.nsBindings.items())
-        if not prefixes:
-            return query
-        return '\n'.join([
-            '\n'.join(['PREFIX %s: <%s>' % (k, v) for k, v in prefixes]),
-            '',  # separate prefixes from query with an empty line
-            query
-        ])
-
+from six import string_types
 
 BNODE_IDENT_PATTERN = re.compile('(?P<label>_\:[^\s]+)')
-SPARQL_NS = Namespace('http://www.w3.org/2005/sparql-results#')
-sparqlNsBindings = {'sparql': SPARQL_NS}
-etree_register_namespace("sparql", SPARQL_NS)
-
-
-def _node_from_result(node):
-    """
-    Helper function that casts XML node in SPARQL results
-    to appropriate rdflib term
-    """
-    if node.tag == '{%s}bnode' % SPARQL_NS:
-        return BNode(node.text)
-    elif node.tag == '{%s}uri' % SPARQL_NS:
-        return URIRef(node.text)
-    elif node.tag == '{%s}literal' % SPARQL_NS:
-        value = node.text if node.text is not None else ''
-        if 'datatype' in node.attrib:
-            dt = URIRef(node.attrib['datatype'])
-            return Literal(value, datatype=dt)
-        elif '{http://www.w3.org/XML/1998/namespace}lang' in node.attrib:
-            return Literal(value, lang=node.attrib[
-                "{http://www.w3.org/XML/1998/namespace}lang"])
-        else:
-            return Literal(value)
-    else:
-        raise Exception('Unknown answer type')
-
-
-def CastToTerm(node):
-    warnings.warn(
-        "Call to deprecated function CastToTerm, use _node_from_result.",
-        category=DeprecationWarning,
-    )
-    return _node_from_result(node)
 
 
 def _node_to_sparql(node):
@@ -116,72 +37,10 @@ def _node_to_sparql(node):
     return node.n3()
 
 
+class SPARQLStore(SPARQLConnector, Store):
+    """An RDFLib store around a SPARQL endpoint
 
-def _traverse_sparql_result_dom(
-        doc, as_dictionary=False, node_from_result=_node_from_result):
-    """
-    Returns a generator over tuples of results
-    """
-    # namespace handling in elementtree xpath sub-set is not pretty :(
-    vars_ = [
-        Variable(v.attrib["name"])
-        for v in doc.findall(
-            './{http://www.w3.org/2005/sparql-results#}head/'
-            '{http://www.w3.org/2005/sparql-results#}variable'
-        )
-    ]
-    for result in doc.findall(
-            './{http://www.w3.org/2005/sparql-results#}results/'
-            '{http://www.w3.org/2005/sparql-results#}result'):
-        curr_bind = {}
-        values = []
-        for binding in result.findall(
-                '{http://www.w3.org/2005/sparql-results#}binding'):
-            var_val = binding.attrib["name"]
-            var = Variable(var_val)
-            term = node_from_result(binding.findall('*')[0])
-            values.append(term)
-            curr_bind[var] = term
-        if as_dictionary:
-            yield curr_bind, vars_
-        else:
-            def __locproc(values_):
-                if len(values_) == 1:
-                    return values_[0]
-                else:
-                    return tuple(values_)
-            yield __locproc(values), vars_
-
-
-def TraverseSPARQLResultDOM(doc, asDictionary=False):
-    warnings.warn(
-        "Call to deprecated function TraverseSPARQLResultDOM, use "
-        "_traverse_sparql_result_dom instead and update asDictionary arg to "
-        "as_dictionary.",
-        category=DeprecationWarning,
-    )
-    return _traverse_sparql_result_dom(
-        doc, as_dictionary=asDictionary, node_from_result=_node_from_result)
-
-
-def _local_name(qname):
-    # wtf - elementtree cant do this for me
-    return qname[qname.index("}") + 1:]
-
-
-def localName(qname):
-    warnings.warn(
-        "Call to deprecated unused function localName, will be dropped soon.",
-        category=DeprecationWarning,
-    )
-    return _local_name(qname)
-
-
-class SPARQLStore(NSSPARQLWrapper, Store):
-    """
-    An RDFLib store around a SPARQL endpoint
-
-    This is in theory context-aware and should work as expected
+    This is context-aware and should work as expected
     when a context is specified.
 
     For ConjunctiveGraphs, reading is done from the "default graph". Exactly
@@ -191,7 +50,7 @@ class SPARQLStore(NSSPARQLWrapper, Store):
     motivated by the SPARQL 1.1.
 
     Fuseki/TDB has a flag for specifying that the default graph
-    is the union of all graphs (tdb:unionDefaultGraph in the Fuseki config).
+    is the union of all graphs (``tdb:unionDefaultGraph`` in the Fuseki config).
 
     .. warning:: By default the SPARQL Store does not support blank-nodes!
 
@@ -201,9 +60,9 @@ class SPARQLStore(NSSPARQLWrapper, Store):
 
                  See http://www.w3.org/TR/sparql11-query/#BGPsparqlBNodes
 
-    You can make use of such extensions through the node_to_sparql and
-    node_from_result arguments. For example if you want to transform
-    BNode('0001') into "<bnode:b0001>", you can use a function like this:
+    You can make use of such extensions through the ``node_to_sparql``
+    argument. For example if you want to transform BNode('0001') into
+    "<bnode:b0001>", you can use a function like this:
 
     >>> def my_bnode_ext(node):
     ...    if isinstance(node, BNode):
@@ -212,6 +71,22 @@ class SPARQLStore(NSSPARQLWrapper, Store):
     >>> store = SPARQLStore('http://dbpedia.org/sparql',
     ...                     node_to_sparql=my_bnode_ext)
 
+    You can request a particular result serialization with the
+    ``returnFormat`` parameter. This is a string that must have a
+    matching plugin registered. Built in is support for ``xml``,
+    ``json``, ``csv``, ``tsv`` and ``application/rdf+xml``.
+
+    The underlying SPARQLConnector builds in the requests library.
+    Any extra kwargs passed to the SPARQLStore connector are passed to
+    requests when doing HTTP calls. I.e. you have full control of
+    cookies/auth/headers.
+
+    Form example:
+
+    >>> store = SPARQLStore('...my endpoint ...', auth=('user','pass'))
+
+    will use HTTP basic auth.
+
     """
     formula_aware = False
     transaction_aware = False
@@ -219,32 +94,22 @@ class SPARQLStore(NSSPARQLWrapper, Store):
     regex_matching = NATIVE_REGEX
 
     def __init__(self,
-                 endpoint=None, bNodeAsURI=False,
+                 endpoint=None,
                  sparql11=True, context_aware=True,
                  node_to_sparql=_node_to_sparql,
-                 node_from_result=_node_from_result,
-                 default_query_method=GET,
-                 **sparqlwrapper_kwargs):
+                 returnFormat='xml',
+                 **sparqlconnector_kwargs):
         """
         """
         super(SPARQLStore, self).__init__(
-            endpoint, returnFormat=XML, **sparqlwrapper_kwargs)
-        self.setUseKeepAlive()
-        self.bNodeAsURI = bNodeAsURI
-        if bNodeAsURI:
-            warnings.warn(
-                "bNodeAsURI argument was never supported and will be dropped "
-                "in favor of node_to_sparql and node_from_result args.",
-                category=DeprecationWarning,
-            )
+            endpoint, returnFormat=returnFormat, **sparqlconnector_kwargs)
+
         self.node_to_sparql = node_to_sparql
-        self.node_from_result = node_from_result
         self.nsBindings = {}
         self.sparql11 = sparql11
         self.context_aware = context_aware
         self.graph_aware = context_aware
-        self._timeout = None
-        self.query_method = default_query_method
+        self._queries = 0
 
     # Database Management Methods
     def create(self, configuration):
@@ -260,15 +125,6 @@ class SPARQLStore(NSSPARQLWrapper, Store):
 
         self.query_endpoint = configuration
 
-    def __set_query_endpoint(self, queryEndpoint):
-        super(SPARQLStore, self).__init__(queryEndpoint, returnFormat=XML)
-        self.endpoint = queryEndpoint
-
-    def __get_query_endpoint(self):
-        return self.endpoint
-
-    query_endpoint = property(__get_query_endpoint, __set_query_endpoint)
-
     def destroy(self, configuration):
         raise TypeError('The SPARQL store is read only')
 
@@ -279,25 +135,43 @@ class SPARQLStore(NSSPARQLWrapper, Store):
     def rollback(self):
         raise TypeError('The SPARQL store is read only')
 
-    def add(self, xxx_todo_changeme, context=None, quoted=False):
-        (subject, predicate, obj) = xxx_todo_changeme
+    def add(self, _, context=None, quoted=False):
         raise TypeError('The SPARQL store is read only')
 
     def addN(self, quads):
         raise TypeError('The SPARQL store is read only')
 
-    def remove(self, xxx_todo_changeme1, context):
-        (subject, predicate, obj) = xxx_todo_changeme1
+    def remove(self, _, context):
         raise TypeError('The SPARQL store is read only')
 
+    def _query(self, *args, **kwargs):
+        self._queries += 1
+
+        return super(SPARQLStore, self).query(*args, **kwargs)
+
+    def _inject_prefixes(self, query, extra_bindings):
+        bindings = list(self.nsBindings.items()) + list(extra_bindings.items())
+        if not bindings:
+            return query
+        return '\n'.join([
+            '\n'.join(['PREFIX %s: <%s>' % (k, v) for k, v in bindings]),
+            '',  # separate ns_bindings from query with an empty line
+            query
+        ])
+
+    def _preprocess_query(self, query):
+        return self._inject_prefixes(query)
+
     def query(self, query,
               initNs={},
               initBindings={},
               queryGraph=None,
               DEBUG=False):
         self.debug = DEBUG
-        assert isinstance(query, str)
-        self.setNamespaceBindings(initNs)
+        assert isinstance(query, string_types)
+
+        query = self._inject_prefixes(query, initNs)
+
         if initBindings:
             if not self.sparql11:
                 raise Exception(
@@ -309,18 +183,10 @@ class SPARQLStore(NSSPARQLWrapper, Store):
                 % (" ".join("?" + str(x) for x in v),
                    " ".join(self.node_to_sparql(initBindings[x]) for x in v))
 
-        self.resetQuery()
-        self.setMethod(self.query_method)
-        if self._is_contextual(queryGraph):
-            self.addParameter("default-graph-uri", queryGraph)
-        self.timeout = self._timeout
-        self.setQuery(query)
+        return self._query(query,
+                           default_graph=queryGraph if self._is_contextual(queryGraph) else None)
 
-        with contextlib.closing(SPARQLWrapper.query(self).response) as res:
-            return Result.parse(res)
-
-
-    def triples(self, xxx_todo_changeme2, context=None):
+    def triples(self, spo, context=None):
         """
         - tuple **(s, o, p)**
             the triple used as filter for the SPARQL select.
@@ -355,7 +221,9 @@ class SPARQLStore(NSSPARQLWrapper, Store):
         del a_graph.OFFSET
         ``
         """
-        (s, p, o) = xxx_todo_changeme2
+
+        s, p, o = spo
+
         vars = []
         if not s:
             s = Variable('s')
@@ -370,11 +238,12 @@ class SPARQLStore(NSSPARQLWrapper, Store):
 
         if vars:
             v = ' '.join([term.n3() for term in vars])
+            verb = 'SELECT %s ' % v
         else:
-            v = '*'
+            verb = 'ASK'
 
         nts = self.node_to_sparql
-        query = "SELECT %s WHERE { %s %s %s }" % (v, nts(s), nts(p), nts(o))
+        query = "%s { %s %s %s }" % (verb, nts(s), nts(p), nts(o))
 
         # The ORDER BY is necessary
         if hasattr(context, LIMIT) or hasattr(context, OFFSET) \
@@ -400,25 +269,19 @@ class SPARQLStore(NSSPARQLWrapper, Store):
         except (ValueError, TypeError, AttributeError):
             pass
 
-        self.resetQuery()
-        if self._is_contextual(context):
-            self.addParameter("default-graph-uri", context.identifier)
-        self.timeout = self._timeout
-        self.setQuery(query)
+        result = self._query(query,
+                             default_graph=context.identifier if self._is_contextual(context) else None)
 
-        with contextlib.closing(SPARQLWrapper.query(self).response) as res:
-            doc = etree.parse(res)
+        if vars:
+            for row in result:
+                yield (row.get(s, s),
+                       row.get(p, p),
+                       row.get(o, o)), None  # why is the context here not the passed in graph 'context'?
+        else:
+            if result.askAnswer:
+                yield (s, p, o), None
 
-        # ElementTree.dump(doc)
-        for rt, vars in _traverse_sparql_result_dom(
-                doc,
-                as_dictionary=True,
-                node_from_result=self.node_from_result):
-            yield (rt.get(s, s),
-                   rt.get(p, p),
-                   rt.get(o, o)), None
-
-    def triples_choices(self, xxx_todo_changeme3, context=None):
+    def triples_choices(self, _, context=None):
         """
         A variant of triples that can take a list of terms instead of a
         single term in any slot.  Stores can implement this to optimize
@@ -426,7 +289,6 @@ class SPARQLStore(NSSPARQLWrapper, Store):
         which will iterate over each term in the list and dispatch to
         triples.
         """
-        (subject, predicate, object_) = xxx_todo_changeme3
         raise NotImplementedError('Triples choices currently not supported')
 
     def __len__(self, context=None):
@@ -435,23 +297,12 @@ class SPARQLStore(NSSPARQLWrapper, Store):
                 "For performance reasons, this is not" +
                 "supported for sparql1.0 endpoints")
         else:
-            self.resetQuery()
             q = "SELECT (count(*) as ?c) WHERE {?s ?p ?o .}"
-            if self._is_contextual(context):
-                self.addParameter("default-graph-uri", context.identifier)
-            self.setQuery(q)
 
-            with contextlib.closing(SPARQLWrapper.query(self).response) as res:
-                doc = etree.parse(res)
+            result = self._query(q,
+                                 default_graph=context.identifier if self._is_contextual(context) else None)
 
-            rt, vars = next(iter(
-                _traverse_sparql_result_dom(
-                    doc,
-                    as_dictionary=True,
-                    node_from_result=self.node_from_result
-                )
-            ))
-            return int(rt.get(Variable("c")))
+            return int(next(iter(result)).c)
 
     def contexts(self, triple=None):
         """
@@ -467,7 +318,6 @@ class SPARQLStore(NSSPARQLWrapper, Store):
         Please note that some SPARQL endpoints are not able to find empty named
         graphs.
         """
-        self.resetQuery()
 
         if triple:
             nts = self.node_to_sparql
@@ -475,18 +325,13 @@ class SPARQLStore(NSSPARQLWrapper, Store):
             params = (nts(s if s else Variable('s')),
                       nts(p if p else Variable('p')),
                       nts(o if o else Variable('o')))
-            self.setQuery('SELECT ?name WHERE { GRAPH ?name { %s %s %s }}' % params)
+            q = 'SELECT ?name WHERE { GRAPH ?name { %s %s %s }}' % params
         else:
-            self.setQuery('SELECT ?name WHERE { GRAPH ?name {} }')
+            q = 'SELECT ?name WHERE { GRAPH ?name {} }'
 
-        with contextlib.closing(SPARQLWrapper.query(self).response) as res:
-            doc = etree.parse(res)
+        result = self._query(q)
 
-        return (
-            rt.get(Variable("name"))
-            for rt, vars in _traverse_sparql_result_dom(
-                doc, as_dictionary=True, node_from_result=self.node_from_result)
-        )
+        return (row.name for row in result)
 
     # Namespace persistence interface implementation
     def bind(self, prefix, namespace):
@@ -495,14 +340,14 @@ class SPARQLStore(NSSPARQLWrapper, Store):
     def prefix(self, namespace):
         """ """
         return dict(
-            [(v, k) for k, v in list(self.nsBindings.items())]
+            [(v, k) for k, v in self.nsBindings.items()]
         ).get(namespace)
 
     def namespace(self, prefix):
         return self.nsBindings.get(prefix)
 
     def namespaces(self):
-        for prefix, ns in list(self.nsBindings.items()):
+        for prefix, ns in self.nsBindings.items():
             yield prefix, ns
 
     def add_graph(self, graph):
@@ -517,11 +362,14 @@ class SPARQLStore(NSSPARQLWrapper, Store):
         """
         if (not self.context_aware) or (graph is None):
             return False
-        if isinstance(graph, str):
+        if isinstance(graph, string_types):
             return graph != '__UNION__'
         else:
             return graph.identifier != DATASET_DEFAULT_GRAPH_ID
 
+    def close(self, commit_pending_transaction=None):
+        SPARQLConnector.close(self)
+
 
 class SPARQLUpdateStore(SPARQLStore):
     """A store using SPARQL queries for reading and SPARQL Update for changes.
@@ -563,22 +411,23 @@ class SPARQLUpdateStore(SPARQLStore):
     # in order to avoid unbalanced curly braces.
 
     # From the SPARQL grammar
-    STRING_LITERAL1 = r"'([^'\\]|\\.)*'"
-    STRING_LITERAL2 = r'"([^"\\]|\\.)*"'
-    STRING_LITERAL_LONG1 = r"'''(('|'')?([^'\\]|\\.))*'''"
-    STRING_LITERAL_LONG2 = r'"""(("|"")?([^"\\]|\\.))*"""'
-    String = '(%s)|(%s)|(%s)|(%s)' % (STRING_LITERAL1, STRING_LITERAL2, STRING_LITERAL_LONG1, STRING_LITERAL_LONG2)
-    IRIREF = r'<([^<>"{}|^`\]\\\[\x00-\x20])*>'
-    COMMENT = r'#[^\x0D\x0A]*([\x0D\x0A]|\Z)'
+    STRING_LITERAL1 = u"'([^'\\\\]|\\\\.)*'"
+    STRING_LITERAL2 = u'"([^"\\\\]|\\\\.)*"'
+    STRING_LITERAL_LONG1 = u"'''(('|'')?([^'\\\\]|\\\\.))*'''"
+    STRING_LITERAL_LONG2 = u'"""(("|"")?([^"\\\\]|\\\\.))*"""'
+    String = u'(%s)|(%s)|(%s)|(%s)' % (STRING_LITERAL1, STRING_LITERAL2,
+                                       STRING_LITERAL_LONG1, STRING_LITERAL_LONG2)
+    IRIREF = u'<([^<>"{}|^`\\]\\\\\[\\x00-\\x20])*>'
+    COMMENT = u'#[^\\x0D\\x0A]*([\\x0D\\x0A]|\\Z)'
 
     # Simplified grammar to find { at beginning and } at end of blocks
-    BLOCK_START = '{'
-    BLOCK_END = '}'
-    ESCAPED = r'\\.'
+    BLOCK_START = u'{'
+    BLOCK_END = u'}'
+    ESCAPED = u'\\\\.'
 
     # Match anything that doesn't start or end a block:
-    BlockContent = '(%s)|(%s)|(%s)|(%s)' % (String, IRIREF, COMMENT, ESCAPED)
-    BlockFinding = '(?P<block_start>%s)|(?P<block_end>%s)|(?P<block_content>%s)' % (BLOCK_START, BLOCK_END, BlockContent)
+    BlockContent = u'(%s)|(%s)|(%s)|(%s)' % (String, IRIREF, COMMENT, ESCAPED)
+    BlockFinding = u'(?P<block_start>%s)|(?P<block_end>%s)|(?P<block_content>%s)' % (BLOCK_START, BLOCK_END, BlockContent)
     BLOCK_FINDING_PATTERN = re.compile(BlockFinding)
 
     # Note that BLOCK_FINDING_PATTERN.finditer() will not cover the whole
@@ -587,46 +436,57 @@ class SPARQLUpdateStore(SPARQLStore):
 
     ##################################################################
 
-
     def __init__(self,
                  queryEndpoint=None, update_endpoint=None,
-                 bNodeAsURI=False, sparql11=True,
+                 sparql11=True,
                  context_aware=True,
-                 postAsEncoded=True, autocommit=True,
+                 postAsEncoded=True,
+                 autocommit=True,
+                 dirty_reads=False,
                  **kwds
                  ):
+        """
+        :param autocommit if set, the store will commit after every
+        writing operations. If False, we only make queries on the
+        server once commit is called.
+
+        :param dirty_reads if set, we do not commit before reading. So you
+        cannot read what you wrote before manually calling commit.
+
+        """
 
         SPARQLStore.__init__(
             self,
             queryEndpoint,
-            bNodeAsURI,
             sparql11,
             context_aware,
-            updateEndpoint=update_endpoint,
+            update_endpoint=update_endpoint,
             **kwds
         )
 
         self.postAsEncoded = postAsEncoded
         self.autocommit = autocommit
+        self.dirty_reads = dirty_reads
         self._edits = None
+        self._updates = 0
 
     def query(self, *args, **kwargs):
-        if not self.autocommit:
+        if not self.autocommit and not self.dirty_reads:
             self.commit()
         return SPARQLStore.query(self, *args, **kwargs)
 
     def triples(self, *args, **kwargs):
-        if not self.autocommit:
+        if not self.autocommit and not self.dirty_reads:
             self.commit()
         return SPARQLStore.triples(self, *args, **kwargs)
 
     def contexts(self, *args, **kwargs):
-        if not self.autocommit:
+        if not self.autocommit and not self.dirty_reads:
             self.commit()
         return SPARQLStore.contexts(self, *args, **kwargs)
 
     def __len__(self, *args, **kwargs):
-        if not self.autocommit:
+        if not self.autocommit and not self.dirty_reads:
             self.commit()
         return SPARQLStore.__len__(self, *args, **kwargs)
 
@@ -642,32 +502,20 @@ class SPARQLUpdateStore(SPARQLStore):
             raise Exception("Cannot create a SPARQL Endpoint")
 
         if isinstance(configuration, tuple):
-            self.endpoint = configuration[0]
+            self.query_endpoint = configuration[0]
             if len(configuration) > 1:
-                self.updateEndpoint = configuration[1]
+                self.update_endpoint = configuration[1]
         else:
-            self.endpoint = configuration
+            self.query_endpoint = configuration
 
-        if not self.updateEndpoint:
-            self.updateEndpoint = self.endpoint
+        if not self.update_endpoint:
+            self.update_endpoint = self.endpoint
 
     def _transaction(self):
         if self._edits is None:
             self._edits = []
         return self._edits
 
-    def __set_update_endpoint(self, update_endpoint):
-        self.updateEndpoint = update_endpoint
-
-    def __get_update_endpoint(self):
-        return self.updateEndpoint
-
-    update_endpoint = property(
-        __get_update_endpoint,
-        __set_update_endpoint,
-        doc='the HTTP URL for the Update endpoint, typically '
-            'something like http://server/dataset/update')
-
     # Transactional interfaces
     def commit(self):
         """ add(), addN(), and remove() are transactional to reduce overhead of many small edits.
@@ -676,7 +524,7 @@ class SPARQLUpdateStore(SPARQLStore):
             and reads can degenerate to the original call-per-triple situation that originally existed.
         """
         if self._edits and len(self._edits) > 0:
-            self._do_update('\n;\n'.join(self._edits))
+            self._update('\n;\n'.join(self._edits))
             self._edits = None
 
     def rollback(self):
@@ -685,7 +533,7 @@ class SPARQLUpdateStore(SPARQLStore):
     def add(self, spo, context=None, quoted=False):
         """ Add a triple to the store of triples. """
 
-        if not self.endpoint:
+        if not self.update_endpoint:
             raise Exception("UpdateEndpoint is not set - call 'open'")
 
         assert not quoted
@@ -704,12 +552,12 @@ class SPARQLUpdateStore(SPARQLStore):
 
     def addN(self, quads):
         """ Add a list of quads to the store. """
-        if not self.endpoint:
+        if not self.update_endpoint:
             raise Exception("UpdateEndpoint is not set - call 'open'")
 
         contexts = collections.defaultdict(list)
         for subject, predicate, obj, context in quads:
-            contexts[context].append((subject,predicate,obj))
+            contexts[context].append((subject, predicate, obj))
         data = []
         nts = self.node_to_sparql
         for context in contexts:
@@ -726,7 +574,7 @@ class SPARQLUpdateStore(SPARQLStore):
 
     def remove(self, spo, context):
         """ Remove a triple from the store """
-        if not self.endpoint:
+        if not self.update_endpoint:
             raise Exception("UpdateEndpoint is not set - call 'open'")
 
         (subject, predicate, obj) = spo
@@ -741,9 +589,7 @@ class SPARQLUpdateStore(SPARQLStore):
         triple = "%s %s %s ." % (nts(subject), nts(predicate), nts(obj))
         if self._is_contextual(context):
             cid = nts(context.identifier)
-            q = "DELETE { GRAPH %s { %s } } WHERE { GRAPH %s { %s } }" % (
-                cid, triple,
-                cid, triple)
+            q = "WITH %(graph)s DELETE { %(triple)s } WHERE { %(triple)s }" % { 'graph': cid, 'triple': triple }
         else:
             q = "DELETE { %s } WHERE { %s } " % (triple, triple)
         self._transaction().append(q)
@@ -753,20 +599,11 @@ class SPARQLUpdateStore(SPARQLStore):
     def setTimeout(self, timeout):
         self._timeout = int(timeout)
 
-    def _do_update(self, update):
-        self.resetQuery()
-        self.setQuery(update)
-        self.setMethod(POST)
-        self.timeout = self._timeout
-        self.setRequestMethod(URLENCODED if self.postAsEncoded else POSTDIRECTLY)
+    def _update(self, update):
 
-        result = SPARQLWrapper.query(self)
+        self._updates += 1
 
-        # we must read (and discard) the whole response
-        # otherwise the network socket buffer will at some point be "full"
-        # and we will block
-        with contextlib.closing(result.response) as res:
-            res.read()
+        SPARQLConnector.update(self, update)
 
     def update(self, query,
                initNs={},
@@ -805,13 +642,12 @@ class SPARQLUpdateStore(SPARQLStore):
               uncommon situations and produce invalid output.
 
         """
-        if not self.endpoint:
+        if not self.update_endpoint:
             raise Exception("UpdateEndpoint is not set - call 'open'")
 
         self.debug = DEBUG
-        assert isinstance(query, str)
-        self.setNamespaceBindings(initNs)
-        query = self.injectPrefixes(query)
+        assert isinstance(query, string_types)
+        query = self._inject_prefixes(query, initNs)
 
         if self._is_contextual(queryGraph):
             query = self._insert_named_graph(query, queryGraph)
@@ -895,6 +731,13 @@ class SPARQLUpdateStore(SPARQLStore):
             self.update(
                 "CREATE GRAPH %s" % self.node_to_sparql(graph.identifier))
 
+    def close(self, commit_pending_transaction=False):
+
+        if commit_pending_transaction:
+            self.commit()
+
+        super(SPARQLStore, self).close()
+
     def remove_graph(self, graph):
         if not self.graph_aware:
             Store.remove_graph(self, graph)
